@@ -1,9 +1,7 @@
 // -*- c++ -*-
-/* $Id: main.cc,v 1.5 2002/04/10 05:55:37 daniel Exp $ */
+/* $Id: main.cc,v 1.8 2002/04/23 10:43:22 daniel Exp $ */
 
-/* main.cc
- *
- * Copyright (C) 2002 The gtkmm Development Team
+/* Copyright (C) 2002 Free Software Foundation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +21,7 @@
 #include <glibmm/main.h>
 #include <glibmm/exceptionhandler.h>
 #include <glibmm/thread.h>
+#include <glibmm/wrap.h>
 #include <glib/gmessages.h>
 
 
@@ -112,6 +111,7 @@ SigC::SlotNode* SourceConnectionNode::get_slot_node()
   return static_cast<SigC::SlotNode*>(slot().impl());
 }
 
+
 /* Glib::Source doesn't use the callback function installed with
  * g_source_set_callback().  Instead, it invokes the sigc++ slot
  * directly from dispatch_vfunc(), which is both simpler and more
@@ -151,6 +151,30 @@ gboolean glibmm_source_callback(void* data)
 namespace Glib
 {
 
+/**** Glib::PollFD *********************************************************/
+
+PollFD::PollFD()
+{
+  gobject_.fd      = 0;
+  gobject_.events  = 0;
+  gobject_.revents = 0;
+}
+
+PollFD::PollFD(int fd)
+{
+  gobject_.fd      = fd;
+  gobject_.events  = 0;
+  gobject_.revents = 0;
+}
+
+PollFD::PollFD(int fd, IOCondition events)
+{
+  gobject_.fd      = fd;
+  gobject_.events  = events;
+  gobject_.revents = 0;
+}
+
+
 /**** Glib::SignalTimeout **************************************************/
 
 inline
@@ -222,7 +246,7 @@ SignalIdle signal_idle()
 }
 
 
-/**** Glib::SignalIO *****************************************************/
+/**** Glib::SignalIO *******************************************************/
 
 inline
 SignalIO::SignalIO(GMainContext* context)
@@ -262,9 +286,7 @@ Glib::RefPtr<MainContext> MainContext::create()
 // static
 Glib::RefPtr<MainContext> MainContext::get_default()
 {
-  GMainContext *const gobject = g_main_context_default();
-  g_main_context_ref(gobject);
-  return Glib::RefPtr<MainContext>(reinterpret_cast<MainContext*>(gobject));
+  return Glib::wrap(g_main_context_default(), true);
 }
 
 bool MainContext::iteration(bool may_block)
@@ -277,6 +299,11 @@ bool MainContext::pending()
   return g_main_context_pending(gobj());
 }
 
+void MainContext::wakeup()
+{
+  g_main_context_wakeup(gobj());
+}
+
 bool MainContext::acquire()
 {
   return g_main_context_acquire(gobj());
@@ -304,11 +331,14 @@ bool MainContext::prepare()
 
 void MainContext::query(int max_priority, int& timeout, std::vector<PollFD>& fds)
 {
+  if(fds.empty())
+    fds.resize(8); // rather bogus number, but better than 0
+
   for(;;)
   {
     const int size_before = fds.size();
     const int size_needed = g_main_context_query(
-        gobj(), max_priority, &timeout, &fds[0], size_before);
+        gobj(), max_priority, &timeout, fds.front().gobj(), size_before);
 
     fds.resize(size_needed);
 
@@ -319,7 +349,10 @@ void MainContext::query(int max_priority, int& timeout, std::vector<PollFD>& fds
 
 bool MainContext::check(int max_priority, std::vector<PollFD>& fds)
 {
-  return g_main_context_check(gobj(), max_priority, &fds[0], fds.size());
+  if(!fds.empty())
+    return g_main_context_check(gobj(), max_priority, fds.front().gobj(), fds.size());
+  else
+    return false;
 }
 
 void MainContext::dispatch()
@@ -339,12 +372,12 @@ GPollFunc MainContext::get_poll_func()
 
 void MainContext::add_poll(PollFD& fd, int priority)
 {
-  g_main_context_add_poll(gobj(), &fd, priority);
+  g_main_context_add_poll(gobj(), fd.gobj(), priority);
 }
 
 void MainContext::remove_poll(PollFD& fd)
 {
-  g_main_context_remove_poll(gobj(), &fd);
+  g_main_context_remove_poll(gobj(), fd.gobj());
 }
 
 SignalTimeout MainContext::signal_timeout()
@@ -408,7 +441,7 @@ Glib::RefPtr<MainLoop> MainLoop::create(bool is_running)
 Glib::RefPtr<MainLoop> MainLoop::create(const Glib::RefPtr<MainContext>& context, bool is_running)
 {
   return Glib::RefPtr<MainLoop>(
-      reinterpret_cast<MainLoop*>(g_main_loop_new(context->gobj(), is_running)));
+      reinterpret_cast<MainLoop*>(g_main_loop_new(Glib::unwrap(context), is_running)));
 }
 
 void MainLoop::run()
@@ -428,9 +461,7 @@ bool MainLoop::is_running()
 
 Glib::RefPtr<MainContext> MainLoop::get_context()
 {
-  GMainContext *const context = g_main_loop_get_context(gobj());
-  g_main_context_ref(context);
-  return Glib::RefPtr<MainContext>(reinterpret_cast<MainContext*>(context));
+  return Glib::wrap(g_main_loop_get_context(gobj()), true);
 }
 
 void MainLoop::reference() const
@@ -483,7 +514,7 @@ GSourceFuncs Source::vfunc_table_ =
 
 unsigned int Source::attach(const Glib::RefPtr<MainContext>& context)
 {
-  return g_source_attach(gobject_, context->gobj());
+  return g_source_attach(gobject_, Glib::unwrap(context));
 }
 
 unsigned int Source::attach()
@@ -526,7 +557,7 @@ Glib::RefPtr<MainContext> Source::get_context()
   return Glib::wrap(g_source_get_context(gobject_), true);
 }
 
-GSource* Source::gobj_copy()
+GSource* Source::gobj_copy() const
 {
   return g_source_ref(gobject_);
 }
@@ -579,12 +610,12 @@ SigC::Connection Source::connect_generic(const SigC::SlotBase& slot)
 
 void Source::add_poll(Glib::PollFD& poll_fd)
 {
-  g_source_add_poll(gobject_, &poll_fd);
+  g_source_add_poll(gobject_, poll_fd.gobj());
 }
 
 void Source::remove_poll(Glib::PollFD& poll_fd)
 {
-  g_source_remove_poll(gobject_, &poll_fd);
+  g_source_remove_poll(gobject_, poll_fd.gobj());
 }
 
 void Source::get_current_time(Glib::TimeVal& current_time)
@@ -791,11 +822,9 @@ SigC::Connection IOSource::connect(const SigC::Slot1<bool,IOCondition>& slot)
 }
 
 IOSource::IOSource(int fd, IOCondition condition)
+:
+  poll_fd_ (fd, condition)
 {
-  poll_fd_.fd = fd;
-  poll_fd_.events = condition;
-  poll_fd_.revents = 0;
-
   add_poll(poll_fd_);
 }
 
@@ -810,13 +839,13 @@ bool IOSource::prepare(int& timeout)
 
 bool IOSource::check()
 {
-  return((poll_fd_.revents & poll_fd_.events) != 0);
+  return ((poll_fd_.get_revents() & poll_fd_.get_events()) != 0);
 }
 
 bool IOSource::dispatch(SigC::SlotNode* slot_data)
 {
   SigC::Slot1<bool,IOCondition> slot (slot_data);
-  return slot(static_cast<IOCondition>(poll_fd_.revents));
+  return slot(poll_fd_.get_revents());
 }
 
 } // namespace Glib
